% Implements a complete hdr and tonemapping cycle
%
% 1.: Computes the camera response curve according to "Recovering High
% Dynamic Range Radiance Maps from Photographs" by P. Debevec.
% You need a wide range of differently exposed pictures from the same scene
% to get good results.
%
% 2.: Recovers a hdr radiance map from the range of ldr pictures 
%
% 3.: Tonemaps the resulting hdr radiance map according to "Photographic
% Tone Reproduction for Digital Images" by Reinhard et al.
% Both the simple global and the more complex local tonemapping operator
% are applied to the hdr radiance map.


% specify the directory that contains your range of differently exposed
% pictures. See readDir.m for details.
dirName = ('../pics/window/small/');
%dirName = ('../pics/hackeschehoefe/small/');
%dirName = ('../pics/sbahn/small/');
[filenames, exposures, numExposures] = readDir(dirName);


fprintf('Opening test image\n');
tmp = imread(filenames{1});

numPixels = size(tmp,1) * size(tmp,2);
numExposures = size(filenames,2);


% define lamda smoothing factor
l = 50;


fprintf('Computing weighting function\n');
% precompute the weighting function value
% for each pixel
weights = [];
for i=1:256
    weights(i) = weight(i,1,256);
end

% load and sample the images
[zRed, zGreen, zBlue, sampleIndices] = makeImageMatrix(filenames, numPixels);


B = zeros(size(zRed,1)*size(zRed,2), numExposures);

fprintf('Creating exposures matrix B\n')
for i = 1:numExposures
    B(:,i) = log(exposures(i));
end

% solve the system for each color channel
fprintf('Solving for red channel\n')
[gRed,lERed]=gsolve(zRed, B, l, weights);
fprintf('Solving for green channel\n')
[gGreen,lEGreen]=gsolve(zGreen, B, l, weights);
fprintf('Solving for blue channel\n')
[gBlue,lEBlue]=gsolve(zBlue, B, l, weights);
save('gMatrix.mat','gRed', 'gGreen', 'gBlue');


% compute the hdr radiance map
fprintf('Computing hdr image\n')
hdr = hdr(filenames, gRed, gGreen, gBlue, weights, B);


% compute the hdr luminance map from the hdr radiance map. It is needed as
% an input for the Reinhard tonemapping operators.
fprintf('Computing luminance map\n');
luminance = 0.2125 * hdr(:,:,1) + 0.7154 * hdr(:,:,2) + 0.0721 * hdr(:,:,3);

% apply Reinhard local tonemapping operator to the hdr radiance map
fprintf('Tonemapping - Reinhard local operator\n');
saturation = 0.6;
eps = 0.05;
phi = 8;
[ldrLocal, luminanceLocal, v, v1Final, sm ]  = reinhardLocal(hdr, saturation, eps, phi);

% apply Reinhard global tonemapping oparator to the hdr radiance map
fprintf('Tonemapping - Reinhard global operator\n');
% specify resulting brightness of the tonampped image. See reinhardGlobal.m
% for details
a = 0.72;
% specify saturation of the resulting tonemapped image. See reinhardGlobal.m
% for details
saturation = 0.6;
[ldrGlobal, luminanceGlobal ] = reinhardGlobal( hdr, a, saturation );

fprintf('Finished!\n');
